//---------------------------------------------------------------------------
// Copyright (C) 2000 Dallas Semiconductor Corporation, All Rights Reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of Dallas Semiconductor
// shall not be used except as stated in the Dallas Semiconductor
// Branding Policy.
// ---------------------------------------------------------------------------
//
//  temp42.C - Module to read the DS28EA00 - temperature measurement.
//
// ---------------------------------------------------------------------------
//
//
#include "ownet.h"
#include "temp42.h"

//----------------------------------------------------------------------
// Read the temperature of a DS28EA00 (family code 0x42)
//
// 'portnum'     - number 0 to MAX_PORTNUM-1.  This number was provided to
//                 OpenCOM to indicate the port number.
// 'SerialNum'   - Serial Number of DS18B20 to read temperature from
// 'Temp '       - pointer to variable where that temperature will be
//                 returned
//
// Returns: TRUE(1)  temperature has been read and verified
//          FALSE(0) could not read the temperature, perhaps device is not
//                   in contact
//
int ReadTemperature42(int portnum, uchar * SerialNum, int *Temp, int *frac)
{
	uchar rt = FALSE;
	uchar send_block[30], lastcrc8 = 0;
	int send_cnt, i, loop = 0;
	int power = 0;

	int val_int = 0;
	int is_neg = FALSE;

	// set the device serial number to the counter device
	owSerialNum(portnum, SerialNum, FALSE);

	for (loop = 0; loop < 2; loop++) {
		// check if the chip is connected to VDD
		if (owOverdriveAccess(portnum)) {
			owWriteByte(portnum, 0xB4);
			power = owReadByte(portnum);
		}
		// access the device
		if (owOverdriveAccess(portnum)) {
			// send the convert command and if nesessary start power delivery
			// if (power) {
			//    if (!owWriteBytePower(portnum,0x44))
			//       return FALSE;
			// } else {
			if (!owWriteByte(portnum, 0x44))
				return FALSE;
			// }
			// sleep for 1 second
			msDelay(1000);
			// turn off the 1-Wire Net strong pull-up
			if (power) {
				if (owLevel(portnum, MODE_NORMAL) !=
				    MODE_NORMAL)
					return FALSE;
			}
			// access the device
			if (owOverdriveAccess(portnum)) {
				// create a block to send that reads the temperature
				// read scratchpad command
				send_cnt = 0;
				send_block[send_cnt++] = 0xBE;
				// now add the read bytes for data bytes and crc8
				for (i = 0; i < 9; i++)
					send_block[send_cnt++] = 0xFF;

				// now send the block
				if (owBlock
				    (portnum, FALSE, send_block, send_cnt)) {
					// initialize the CRC8
					setcrc8(portnum, 0);
					// perform the CRC8 on the last 8 bytes of packet
					for (i = send_cnt - 9; i < send_cnt;
					     i++)
						lastcrc8 =
						    docrc8(portnum,
							   send_block[i]);

					// verify CRC8 is correct
					if (lastcrc8 == 0x00) {

						// calculate the high-res temperature
						// from twos complement representation
						val_int = send_block[1];	// low byte
						val_int |= send_block[2] << 8;	// high byte
						if (val_int & 0x00001000) {	// check sign for negative
							val_int = ~val_int + 1;
							is_neg = TRUE;
						}

						*Temp = val_int >> 4;	// only integer part
						*frac = val_int & 0x08;	// only 1/2 bit
						// success
						rt = TRUE;
						break;
					}
				}
			}
		}
	}
	// exit overdrive mode
	owSpeed(portnum, MODE_NORMAL);
	// return the result flag rt
	return rt;
}
